import 'dart:convert';

import 'package:ocg_app/ocg_app.dart';
import 'package:flutter/material.dart';
import 'package:ocg_login/src/repository/token.dart';
import 'package:http/http.dart' as http;
import 'package:sms_autofill/sms_autofill.dart';
import '../database/login.dart';

abstract class LoginRepository {
  Future<AppResponse<bool>> sendOTP(
    String mobileNumber,
  );
  Future<AppResponse<bool>> verifyOTP(String mobileNumber, String otp);
  Future<AppResponse<bool>> isLoggedIn();
}

class LoginTestRepository extends LoginRepository {
  final LoginDatabase loginDatabase;

  LoginTestRepository(
    this.loginDatabase,
  );

  @override
  Future<AppResponse<bool>> isLoggedIn() async {
    try {
      AppResponse<AppToken> response = await loginDatabase.getToken();
      debugPrint("Token ${response.data.toJson()}");
      return AppResponse.named(data: response.isSuccess);
    } catch (e) {
      return AppResponse.named(error: "${e.toString()}");
    }
  }

  @override
  Future<AppResponse<bool>> sendOTP(String mobileNumber) {
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> verifyOTP(String mobileNumber, String otp) {
    throw UnimplementedError();
  }
}

class LoginNetworkRepository extends LoginRepository {
  final LoginTokenRepository tokenRepository;
  final http.BaseClient client;

  LoginNetworkRepository(this.tokenRepository, this.client);

  @override
  Future<AppResponse<bool>> sendOTP(String mobileNumber) async {
    String url = "/auth/code/send/";
    try {
      String hash = await SmsAutoFill().getAppSignature;
      var body = {"phone": "$mobileNumber", "device_hash": hash ?? ""};
      debugPrint("Body $body");
      var response = await client.post(url,
          body: jsonEncode(body), headers: ApiConfig.headers);
      return AppResponse.named(
          data: response.statusCode == 200,
          error: jsonDecode(response.body)["err"]);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> verifyOTP(String mobileNumber, String otp) async {
    try {
      String url = "/auth/code/check/";
      var body = {"phone": "$mobileNumber", "code": otp};
      var response = await client.post(url,
          body: jsonEncode(body), headers: ApiConfig.headers);
      var json = jsonDecode(response.body);
      json["userId"] = json["user_id"];
      AppToken appToken = AppToken.fromJson(json);
      await tokenRepository.saveToken(appToken);
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> isLoggedIn() async {
    try {
      String url = "/self";
      AppResponse<AppToken> response = await tokenRepository.getToken();
      debugPrint("Token ${response.data.toJson()}");
      await client.get(
        url,
      );
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
